# Load Packages
library(ggplot2)
library(tidyr)
source("Multiplot.R")

# Fosten and Gutknecht (2018) "Testing Nowcast Monotonicity with Estimated Factors"
# This R code is part 1 of 2 of the empirical application

# This file runs a pseudo out-of-sample nowcasting experiment for real GDP and 6 subcomponents
# The methodology used is discussed in the paper
# It uses a dynamic factor model to make nowcasts, with an AR(1) interpolation to solve the ragged edge problem

# The outputs of this file are:
# 1) The chart of MSFE for all 6 GDP variables in the nowcast and backcast period
# 2) Multiple csv files of prediction errors for various samples and parameter values

# The csv file is loaded in to the next file, which runs the bootstrap methodology



# Load Data ---------------------------------------------------------------

# N.B. Data for this exercise has already been transformed to stationarity
qlydata=read.csv("Data/US Quarterly.csv")
mlydata=read.csv("Data/US Monthly.csv")
calendar=read.csv("Data/Calendar.csv",header = F)

# The calendar data contains information about the data flow in the nowcast and backcast period:

# Column 1 gives the variable code
# Columns 2-3 give an indicator for inclusion in the small or full dataset
# Column 4 gives the group of the variable according to the Data Appendix of McCracken and Ng (2016)
# Column 5 gives the (approximate) release date of the variable in terms of days after the start of the reference quarter
# Column 6 gives the number of the variable
# Column 7 gives gives which month of data has been released, relative to the first month of the nowcast period
# A value of 0 means that the first month of the nowcast period is released
# A value of -1 means the month before the start of the nowcast period is released
# etc.
# Column 8 gives which quarter of the quarterly GDP variable is available at the time of release of the monthly variable
# A value of -1 means last quarter's GDP is available
# A value of -2 means that last quarter's GDP is NOT available
# These are the only two values possible

# Save quarterly and monthly dates
datesq=matrix(qlydata[,1])
qlydata=qlydata[,-1]
datesm=matrix(mlydata[,1])
mlydata=mlydata[,-1]

Nraw=ncol(mlydata)


# Set dataset configuration parmeters

# datawidth=1 for small dataset N=9 and datawidth=2 for full dataset N=62
datawidth=2
# datalength=1 for 1978-2016, datalength=2 for 1984-2016
datalength=1

# Set calendar choices to determine nowcast update frequency

# NCupdate=1 for nowcasts updated at every release point, NCupdate=2 for nowcasts updated once per day
# NCupdate=3 for nowcasts updated 3 times per month (12 times in NC and BC periods)
NCupdate=1

# Define selection matrix and choose only relevant variables from the dataset and calendar releases
selection=matrix(calendar[sapply(matrix(seq(1,ncol(mlydata),1)),function(x) which(calendar[1:ncol(mlydata),6]==x)),1+datawidth]==1)
Nvars=seq(1,ncol(mlydata),1)
mlydata=mlydata[,selection]
calendar=calendar[calendar[,1+datawidth]==1,]

# If variable selection occurs, re-code variable names to match them with their column in the dataset
if (ncol(mlydata)<Nraw){
  calendar[,6]=matrix(rep(rank(calendar[1:ncol(mlydata),6]),length.out=nrow(calendar)))
}

# Also ignore releases which are leading the current quarter of GDP
# This corresponds to any monthly releases with a value greater than 2 in the calendar
calendar=calendar[(calendar[,7]<3),]

# Generate release number based on NCupdate type
if (NCupdate==1){
  release=matrix(sapply(calendar[,5],function(x) which(x==unique(calendar[,5]))))  
}
if (NCupdate==2){
  release=matrix(sapply(floor(calendar[,5]),function(x) which(x==unique(floor(calendar[,5]))))) 
}
if (NCupdate==3){
  release=matrix(sapply(floor(calendar[,5]/10),function(x) which(x==unique(floor(calendar[,5]/10))))) 
}

# Define number of nowcast updates as S
S=max(release)


# Data Properties ---------------------------------------------------------

# T will be the number of quarterly observations. Monthly observations will be 3*T
# This also depends on datalength, which determines whether we use full sample or post-1984

if (datalength==2){
  qlydata=qlydata[datesq>=1984,]
  datesq=as.matrix(datesq[datesq>=1984,])
  mlydata=mlydata[datesm>=1984,]
  datesm=as.matrix(datesm[datesm>=1984,])
}

T=nrow(qlydata)
N=ncol(mlydata)

# Start looping over pi, the out-of-sample split, for the final results
for (pi in c(1,0.5,0.75,(4/3),2)){
  
  # Set sample sizes
  R=round(T/(1+pi))
  P=T-R
  startdateq=datesq[(R+1),]
  
  # Declare maximum number of factors to be considered
  rmax=8
  
  # Number of dependent variables
  Ndepvar=6
  
  # Factor Model Out-of-Sample  ------------------------------------------
  
  # Preallocate matrices for out-of-sample nowcasts, stacked from left to right by dependent variable
  oos=matrix(NA,P,(S*Ndepvar))
  eps=matrix(NA,P,(S*Ndepvar))
  
  # Loop over the different target variables
  for (depvar in 1:Ndepvar){
    
    # Loop over the P nowcast quarters to be predicted
    for (t in 1:P){
      
      # Create set of monthly variables with ragged edge at the beginning of the nowcast quarter
      # Generate balanced panel for the monthly indicators using AR(1) forcasts until the end of the nowcast period
      X=matrix(NA,(3*(R+t)),N)
      
      for (i in 1:N){
        
        RM=3*(R+t-1)+calendar[i,7]
        X[1:RM,calendar[i,6]]=mlydata[1:RM,calendar[i,6]]
        
        # Define number of required forecasts to address ragged edge
        FM=3*(R+t)-RM
        
        # Obtain AR(1) forecasts of variable for the (3*(R+t)-RM) periods to get balanced panel
        Z=cbind(matrix(1,(RM-1),1),X[1:(RM-1),calendar[i,6]])
        betaAR=solve(t(Z)%*%Z)%*%t(Z)%*%X[2:RM,calendar[i,6]]
        
        for (f in 1:FM){
          X[(3*(R+t)-FM+f),calendar[i,6]]=t(betaAR)%*%matrix(c(1,X[(3*(R+t)-FM+f-1),calendar[i,6]]))
        }  
        
        # i loop end
      }
      
      # Loop over S nowcast points within the quarter
      for (i in 1:S){
        
        # Obtain set of released variables
        released=matrix(calendar[release==i,6])
        
        # Define the length of available quarterly expanding window of data based on GDP availability
        # Either of length R+t-1 if prev quarter GDP has already been released, or R+t-2 if not
        RQ=R+t+(calendar[release==i,8])[1]
        
        # Define y vector for forecast variable, including NA's if the last observations are missing
        y=matrix(NA,(R+t-1),1)
        y[1:RQ,]=100*qlydata[1:RQ,depvar]
        
        # Define number of required predictions (backcasts) of target variable in order to make nowcast
        FQ=R+t-1-RQ
        
        # Obtain backcast of y (if required) to complete the window of R+t-1 quarters
        if (R+t-1-RQ>0){
          
          Z=cbind(matrix(1,(RQ-1),1),y[1:(RQ-1),])
          betaAR=solve(t(Z)%*%Z)%*%t(Z)%*%y[2:RQ,]
          
          for (f in 1:FQ){
            y[(R+t-1-FQ+f),]=t(betaAR)%*%matrix(c(1,y[(R+t-1-FQ+f-1),]))
          }  
          
        }
        
        
        # Identify the released monthly variables, and replace the relevant column of matrix X with this data
        # Rebalance the panel
        # N.B. The window length will be 3*(R+t-1)+1 if the first month of the nowcast period has been released (value of 0 in calendar)
        # It will be exactly 3*(R+t-1) if the new release is the last month of the previous quarter, thus 'completing' the window of R+t-1 quarters
        
        for (j in 1:nrow(released)){
          
          RM=3*(R+t-1)+1+calendar[(min(which(release==i))-1+j),7]
          
          # Replace data in released variable 
          X[,released[j]]=matrix(NA,(3*(R+t)),1)
          X[1:RM,released[j]]=mlydata[1:RM,released[j]]
          
          # Define number of required forecasts of bridge variable
          FM=3*(R+t)-RM
          
          
          # Obtain AR(1) forecasts of bridge variable for the (3*(R+t)-RM) periods, if required
          if (3*(R+t)-RM>0){
            
            Z=cbind(matrix(1,(RM-1),1),X[1:(RM-1),released[j]])
            betaAR=solve(t(Z)%*%Z)%*%t(Z)%*%X[2:RM,released[j]]
            
            for (f in 1:FM){
              X[(3*(R+t)-FM+f),released[j]]=t(betaAR)%*%matrix(c(1,X[(3*(R+t)-FM+f-1),released[j]]))
            }  
            
          }
          
          # j loop end
        }

        # Factor Estimation
        
        # Standardise Variables in X to have 0 mean and Variance 1
        Xt=(X-matrix(1,(3*(R+t)),1)%*%(t(as.matrix(colSums(X)/(3*(R+t))))))*(matrix(1,(3*(R+t)),1)%*%(t(as.matrix(sqrt(diag(var(X))))))^(-1))
        
        ############################
        # Principal Components on X'X
        xx=t(Xt)%*%Xt/(N*3*(R+t))
        eig=eigen(xx,symmetric=TRUE)
        eigval=as.matrix(eig$values)
        
        # Use Bai and Ng (2002) criterion to select # factors
        # only for the first recursive window, and for pi=1
        if (t==1 && pi==1 && depvar==1 && i==1){
          S.r=as.matrix((matrix(1,nrow(eigval),1)-cumsum(eigval))[1:rmax,])
          ICp2=log(S.r)+matrix(seq(1,rmax,1))*((N+3*(R+t))/(N*3*(R+t)))*log(min(N,3*(R+t)))
          r=which.min(ICp2)
        }
        
        Lambdahat=as.matrix(eig$vectors[,1:r])
        
        # Normalise Lambda'Lambda/N=I
        Lambdahat=Lambdahat*((matrix(1,N,1)%*%t(as.matrix(sqrt(colMeans(Lambdahat^2)))))^(-1))
        Fhat=Xt%*%Lambdahat/(N)
        
        # Run (MIDAS) regression of y_t on y_t-1 and skip-sampled Fhat_t. 
        # Matrix Z contains y_{t-1}, F_{t+2/3},F_{t+1/3},F_{t}
        # Only observations up to RQ can be used for estimation
        Z=cbind(matrix(1,(RQ-1),1),y[1:(RQ-1),],as.matrix(Fhat[4:(3*RQ),])[(3*seq(1,(RQ-1))-2),],as.matrix(Fhat[4:(3*RQ),])[(3*seq(1,(RQ-1))-1),],as.matrix(Fhat[4:(3*RQ),])[(3*seq(1,(RQ-1))),])
        beta=solve(t(Z)%*%Z)%*%t(Z)%*%y[2:RQ,]
        
        # Make nowcast of y_t and store in oos. This uses the forecast values of X_t and y_t-1 if it is not yet available
        oos[t,(S*(depvar-1)+i)]=t(beta)%*%matrix(c(1,y[(R+t-1),],Fhat[(3*(R+t-1)+1),],Fhat[(3*(R+t-1)+2),],Fhat[(3*(R+t-1)+3),]))
        
        # i loop end    
      }
      
      
      
      # t loop end  
    }
    
    # Store nowcast errors
    eps[,(S*(depvar-1)+1):(S*depvar)]=100*qlydata[(T-P+1):T,depvar]%*%matrix(1,1,S)-oos[,(S*(depvar-1)+1):(S*depvar)]
    
    # depvar loop end
  }
  
  
  # Process Results, Chart and Save Results-----------------------------------------------
  
  eps2=eps^2
  
  MSFE=matrix(colMeans(eps2))
  
  # MSFE Chart
  path=paste("Results/MSFE Graphs/MSFE-Recursive-",c("AllRelease","DailyRelease","10DayRelease")[NCupdate],"-",round(pi,1),"-",c("Small","Large")[datawidth],"-",c("All","Post84")[datalength],".pdf",sep="")
  pdf(path,width=10,height=6)
  plots=list()
  
  # Define backcast point for plot graphics
  BC=release[min(which(calendar[,5]>89.9))]
  
  for (depvar in 1:Ndepvar){
    
    df=data.frame(cbind(matrix(seq(1,S,1)),MSFE[(S*(depvar-1)+1):(S*depvar),]))
    names(df)=c("i","Factor")
    dfg=gather(df,Method,Data,Factor)
    graph=ggplot(data = dfg, aes(x = i, y = Data, colour = Method)) + 
      geom_line(size = 1) + xlab("Horizon") + ylab("MSFE")  + ylim(c(floor(100*min(MSFE[(S*(depvar-1)+1):(S*depvar),]))/100,ceiling(100*max(MSFE[(S*(depvar-1)+1):(S*depvar),]))/100)) + theme(legend.position="none")
    graph=graph+ggtitle(matrix(c("Aggregate GDP","Consumption","Investment","Government Spending","Exports","Imports"))[depvar,])
    graph=graph+geom_vline(xintercept = BC,linetype="longdash")+annotate("text", x = BC-1, y = (ceiling(100*max(MSFE[(S*(depvar-1)+1):(S*depvar),]))/100), label = "Nowcast",hjust=1)+annotate("text", x = BC+1, y = (ceiling(100*max(MSFE[(S*(depvar-1)+1):(S*depvar),]))/100), label = "Backcast",hjust=0)
    
    plots[[depvar]]=graph
    
  }
  
  multiplot(plotlist=plots,cols=2)
  dev.off()
  
  # Save csv of nowcast errors (for the 6 dependent variables)
  path=paste("Results/Out of Sample/eps-Recursive-",c("AllRelease","DailyRelease","10DayRelease")[NCupdate],"-",round(pi,1),"-",c("Small","Large")[datawidth],"-",c("All","Post84")[datalength],".csv",sep="")
  write.table(eps,path, row.names=FALSE, col.names=FALSE, sep=",")
  
  # Save csv of nowcasts (for the 6 dependent variables)
  path=paste("Results/Out of Sample/oos-Recursive-",c("AllRelease","DailyRelease","10DayRelease")[NCupdate],"-",round(pi,1),"-",c("Small","Large")[datawidth],"-",c("All","Post84")[datalength],".csv",sep="")
  write.table(oos,path, row.names=FALSE, col.names=FALSE, sep=",")
  
  # pi loop end
}