# Fosten and Gutknecht (2018) "Testing Nowcast Monotonicity with Estimated Factors"
# This R code is part 2 of the empirical application

# This file runs the bootstrap methodology described in the paper
# The output of the file is
# a csv table of results for the bootstrap methodology without inequality selection

# Test Statistic and Bootstrap ---------------------------------------------------------------

# Define bootstrap parameters and preallocate reasults matrices outside of the loops

# Number of dependent variables, pi combinations and inequality combinations
Ndepvar=6
Npi=5
Nineq=5

# Preallocate results table
table=matrix("",(Ndepvar*Npi+1),(5*(Nineq)))
tableselect=matrix(NA,Ndepvar,1)

# Set random number seed
set.seed(1000)

# Matrix of large and small block lengths
rq=matrix(c(0,4,0,5,0,6,0,7,0,8,1,4,1,5,1,6,1,7,1,8,2,4,2,5,2,6,2,7,2,8),15,2,byrow=TRUE)

# Set parameters for the bootstrap methodology

# Number of final bootstrap replications
B=399

# Set parameters for the optimal block length methodology

# Number of (outer) bootstrap iterations for optimal block length to calculate the rejection rate
BRO=400
# Number of (inner) bootstrap iterations for optimal block length to calculate the pseudo critical values
BRI=299

# Nominal size for the optimal block bootstrap procedure
nomsize=0.1
nomsize2=0.04

# Initial Block size to capture the dependence structure of the data
rint=0
qint=8

# Set dataset configuration parameters
# This allows results to be run for the different combinations used in the previous file

# scheme=1 for recursive and sceme=2 for rolling
scheme=1
# datawidth=1 for small dataset N=9 and datawidth=2 for full dataset N=62
datawidth=2
# datalength=1 for 1978-2016, datalength=2 for 1984-2016
datalength=1

# Set calendar choices to determine nowcast update frequency

# NCupdate=1 for nowcasts updated at every release point, NCupdate=2 for nowcasts updated once per day
# NCupdate=3 for nowcasts updated 3 times per month (12 times in NC and BC periods)
NCupdate=1

# Loop over pi
for (picount in 1:Npi){
  
  pi=c(0.5,0.75,1,(4/3),2)[picount]
  
  # Load nowcast error series
  path=paste("Results/Out of Sample/eps-",c("Recursive","Rolling")[scheme],"-",c("AllRelease","DailyRelease","10DayRelease")[NCupdate],"-",round(pi,1),"-",c("Small","Large")[datawidth],"-",c("All","Post84")[datalength],".csv",sep="")
  eps=read.csv(path,header=FALSE)
  
  # Define the number of nowcasts per quarter and the number of predictions
  S=ncol(eps)/Ndepvar
  P=nrow(eps)
  
  # Loop over dependent variables
  for (depvar in 1:Ndepvar){
    
    # Generate the moment conditions upon which the test statistic is based
    # Note that these are MSFE differentials
    # As in paper, k_S^c is used to separate inequalities some distance apart
    
    # Set ineq = 1,...,5 where:
    # 1 = full set of inequalities
    # 2 = k_S^c=1
    # 3 = k_S^c=2
    # 4 = k_S^c=5
    # 5 = P&T Delta^e (adjacent-only)

    for (ineq in 1:Nineq){
      
      # Obtain squared error series for relevant depvar
      ff=(eps[,(S*(depvar-1)+1):(S*depvar)])^2
      
      if (ineq<5){
        f=matrix(NA,P,0)
        spacing=c(0,1,2,5)[ineq]
        for (j in (1+spacing):(ncol(ff)-1)){
          f=cbind(f,(ff[,(1+j):ncol(ff)]-ff[,1:(ncol(ff)-j)]))
        }
      }
      
      # Adjacent-only comparisons
      if (ineq==5){
        f=ff[,2:ncol(ff)]-ff[,1:(ncol(ff)-1)]
      } 
      
      # Define number of moment inequalities
      kappa=ncol(f)
      
      # Generate means fbar
      fbar=matrix(colMeans(f),1,kappa)
      
      # Test statistic
      U=sqrt(P)*max(fbar)
      
      # Begin Optimal Block Length Procedure
      
      # Results matrix preallocation
      results=matrix(0,nrow(rq),1)
      
      # Final Block Choice
      blocksfinal=cbind(rq,matrix(0,nrow(rq),1))
      
      # Define "mint": number of blocks of initial blocksize
      mint=floor(P/(rint+qint))
      
      # Pre-define vector of row indices to use for inner bootstrap resampling
      scl=matrix(1:(qint+rint),(qint+rint),1)
      scl=matrix(rep(scl, mint),nrow=length(scl)*mint,ncol=1)
      
      # Initiate the Optimal Blocksize procedure (inner bootstrap) --------------
      
      # Block bootstrap to regenerate the data 	
      for (bbb in 1:BRO){
        
        # Preallocate temporary matrices for the pseudo sample	
        temp=matrix(0,(qint+rint)*mint,kappa)
        
        # Generate m pseudo random integers from 1,...,mint with replacement
        Bindex=sample(0:(mint-1),mint,replace=T)
        
        # Construct filter to extract the randomly drawn blocks
        # And add to the scl vector calculated above
        # Note: filter extracts much faster than any loop
        Bindex=matrix(rep(Bindex*(rint+qint),each=(rint+qint)),nrow=(mint*(rint+qint)),1)
        Bindex=Bindex + scl
        
        # Extract blocks
        pseudof=f[Bindex,]
        
        # Construct Pseudo Test Statistic ----------------------------------------------------------
        
        # Pseudo mean (for bootstrap iteration)
        pseudofbar=matrix(colMeans(pseudof),1,kappa)
        
        # Pseudo Test statistic
        pseudoU=sqrt(P)*max(pseudofbar-fbar)
        
        # "Inner" bootstrap to construct pseudo critical values
        
        # Preallocate vector of non-studentized and studentized bootstrap statistics, across all the combinations of block length
        boot=matrix(0,BRI,nrow(rq))
        
        for (bb in 1:nrow(rq)){
          
          r=rq[bb,1]
          q=rq[bb,2]
          
          # mpseudo taken to give largest integer below P of the form m*(r+q)
          mpseudo=floor(nrow(pseudof)/(r+q))
          
          # Preallocate temporary matrices for the large blocks for fN,fA1,fA2,fA3
          temp2=matrix(0,(q*mpseudo),kappa)
          
          # Small block/large block bootstrap forms blocks of length (q+r) and drops the last small block of length r
          # In each block of length (q+r) from pseudof
          # The mean of the original variables is subtracted prior to multiplying by the random draw	
          
          # Extract blocks from the matrices pseudof	into temp.
          # This is used to create the bootstrapped pseudof in the loop over b
          for (i in 1:mpseudo){
            temp2[((i-1)*q+1):(i*q),]=(pseudof[((i-1)*(q+r)+1):(i*(q+r)-r),]-matrix(pseudofbar,q,kappa,byrow=TRUE))
          }
          
          # Bootstrap resampling
          for (b in 1:BRI){
            
            # Draw an (mpseudo x 1) random i.i.d.Normal vector for multiplier bootstrap
            random=matrix(rnorm(mpseudo,0,1),mpseudo,1)	
            Eps=matrix(rep(random,each=q),nrow=length(rep(random,each=q)),ncol=kappa,byrow=FALSE) 	
            
            # The (q*m)-matrix extracted above is pre-multiplied by the random normal draw contained in the (q*m)x(kappa) matrix Eps	
            fstar=Eps*temp2
            
            # Bootstrap pseudofbar
            fbarstar=matrix(colMeans(fstar),1,kappa)
            
            # Bootstrap max statistics
            boot[b,((bb-1)+1):(bb)]=sqrt(mpseudo*q)*matrix(apply(fbarstar,1,max),1,1)
            
            # Bootstrap b loop end
          }
          
          # Block combinations bb loop end
        }
        
        for (j in 1:nrow(rq)){
          results[j,1]=results[j,1]+(pseudoU<matrix(sort(boot[,j]))[((1-nomsize)*(BRI+1)),])
        }
        
        # Counter
        if(bbb==1){
          cat(paste("/// Progress: depvar ",depvar," of ",6," ",sep=""))
        } else{}
        if (BRO>1){
          if ((bbb %in% seq(10,10*floor(BRO/10),10))==TRUE){
            cat(bbb)
          } else{
            cat('.')
          }
        }
        
        # Bootstrap bbb loop end
      }
      
      # Take average over all BRO replications to obtain rejection probability
      results=results/BRO
      
      # Choose block pair (r,q) with empirical coverage closest to (1-nomsize)%.
      finblock=cbind(rq,blocksfinal[,3],results)
      finblock=finblock[order(abs(finblock[,4]-(1-nomsize)),decreasing=FALSE),]
      rfinal=finblock[1,1]
      qfinal=finblock[1,2]
      covfinal=finblock[1,4]
      
      # Preallocate vector of bootstrap statistics, across the single chosen block length combination
      rq2=t(matrix(c(rfinal,qfinal)))
      boot=matrix(0,B,nrow(rq2))
      
      for (bb in 1:nrow(rq2)){
        
        r=rq2[bb,1]
        q=rq2[bb,2]
        
        # m taken to give largest integer below P of the form m*(r+q)
        m=floor(P/(r+q))
        
        # Preallocate temporary matrices for the large blocks for f
        temp=matrix(0,(q*m),kappa)
        
        # Small block/large block bootstrap forms blocks of length (q+r) and drops the last small block of length r
        # In each block of length (q+r) from f
        # The mean of the original variables is subtracted prior to multiplying by the random draw	
        
        # Extract blocks from the matrix f into temp. 
        # This is used to create the bootstrapped f in the loop over b
        for (i in 1:m){
          temp[((i-1)*q+1):(i*q),]=(f[((i-1)*(q+r)+1):(i*(q+r)-r),]-matrix(fbar[1,],q,kappa,byrow=TRUE))
        }
        
        
        # Bootstrap resampling
        for (b in 1:B){  
          
          # Draw an (m x 1) random i.i.d.Normal vector for multiplier bootstrap
          random=matrix(rnorm(m,0,1),m,1)	     
          Eps=matrix(rep(random,each=q),nrow=length(rep(random,each=q)),ncol=kappa,byrow=FALSE) 	
          
          # The (q*m)-matrix extracted above is pre-multiplied by the random normal draw contained in the (q*m)x(kappa) matrix Eps	
          fstar = Eps*temp
          
          # Bootstrap fbar
          fbarstar=matrix(colMeans(fstar),1,kappa)
          
          # Bootstrap max statistics  
          boot[b,bb]=sqrt(m*q)*max(fbarstar)
          
          
          # Bootstrap b loop end
        }
        
        # Block combinations bb loop end
      }
      
      table[1,(5*(ineq-1)+3)]=paste("kappa=",kappa,sep="")
      table[(Ndepvar*(picount-1)+depvar+1),(5*(ineq-1)+1)]=U
      table[(Ndepvar*(picount-1)+depvar+1),(5*(ineq-1)+2)]=matrix(sort(boot))[((B+1)/2),]
      table[(Ndepvar*(picount-1)+depvar+1),(5*(ineq-1)+3)]=matrix(sort(boot))[(0.9*(B+1)),]
      table[(Ndepvar*(picount-1)+depvar+1),(5*(ineq-1)+4)]=matrix(sort(boot))[(0.95*(B+1)),]
      table[(Ndepvar*(picount-1)+depvar+1),(5*(ineq-1)+5)]=sum(boot>U)/B
      
      if (ineq==1){
        tableselect[depvar,1]=matrix(sort(boot))[((1-nomsize2)*(B+1)),]
      }
      
      # ineq loop end
    }
    
    # depvar loop end
  }
  
  path=paste("Results/Out of Sample/Selection-",c("Recursive","Rolling")[scheme],"-",c("AllRelease","DailyRelease","10DayRelease")[NCupdate],"-",round(pi,1),"-",c("Small","Large")[datawidth],"-",c("All","Post84")[datalength],".csv",sep="")
  write.table(tableselect,path, row.names=FALSE, col.names=FALSE, sep=",")
  
  # picount loop end
}

tablefinal=rbind(table[1,],rep(c("Ustar","50%","90%","95%","p-value"),Nineq),table[2:nrow(table),])
tablefinal=cbind(matrix("",nrow(tablefinal),1),c("","",rep(c("Aggregate GDP","Consumption","Investment","Government Spending","Exports","Imports"),Npi)),tablefinal)

if (datalength==2){
  tablefinal[(Ndepvar*seq(1,Npi,1)-1),1]=paste("R=",c(86,74,65,55,43),sep="")
  tablefinal[(Ndepvar*seq(1,Npi,1)),1]=paste("P=",c(43,55,64,74,86),sep="")
} else {
  tablefinal[(Ndepvar*seq(1,Npi,1)-1),1]=paste("R=",c(101,87,76,65,51),sep="")
  tablefinal[(Ndepvar*seq(1,Npi,1)),1]=paste("P=",c(51,65,76,87,101),sep="")
}

path=paste("Results/Results-",c("Recursive","Rolling")[scheme],"-",c("AllRelease","DailyRelease","10DayRelease")[NCupdate],"-",c("Small","Large")[datawidth],"-",c("All","Post84")[datalength],".csv",sep="")
write.table(tablefinal,path, row.names=FALSE, col.names=FALSE, sep=",")