# Fosten and Gutknecht (2018) "Testing Nowcast Monotonicity with Estimated Factors"
# This R code performs the Monte Carlo simulation study of Section 4.1

# The output of this file is a csv table of rejection frequencies


# Initialise Parameters 

# Set random number seed and get start time
set.seed(1000)
time=proc.time()

# Sample sizes 

# Total time series sample size T
T=300
# Matrix of sample splits pi where P/R=pi
PP=matrix(c(0.5,2),2,1)
# Matrix of number of nowcast models
SS=matrix(c(5,10),2,1)

# Nominal Size
nomsize=0.05

# Number of different types of inequality sets (see below for description) and number of DGPs
Nineq=5
NDGP=3

# Bootstrap parameters

# Matrix of large and small block sizes, q and r, and number of bootstrap replications, B
# Implied no. blocks, m, defined in the loop as it depends on q and r
rq=matrix(c(0,4,0,5,0,6,0,7,0,8,1,4,1,5,1,6,1,7,1,8,2,4,2,5,2,6,2,7,2,8),15,2,byrow=TRUE)

# Number of Monte Carlo replications (B=1 bootstrap draw by Warp speed bootstrap)
M=999

# Begin looping over different inequality combination types for the final table
for (ineq in 1:Nineq){
  
  # Pre-allocate results matrix
  results=matrix(0,(nrow(PP)*nrow(rq)),(nrow(SS)*NDGP))

  # Loop over sample sizes P
  for (pp in 1:nrow(PP)){
    
    # Loop over number of inequalities S
    for (ss in 1:nrow(SS)){
      
      # Set sample sizes, in-sample (R) and out-of-sample (P) and number of variables (N)
      pi=PP[pp]
      R=round(T/(1+pi))
      P=T-R
      S=SS[ss]
      
      # Set DGP parameters which depend on S
      
      # Model for the DGP in Section 4.1
      # y_t=gamma'Z_t+epsilon_t
      # There are three versions of gamma under DGPZ-N, DGPZ-A1 and DGPZ-A2
      gamma=matrix(0,S,3)
      gamma[,1]=1
      gamma[,2]=0
      gamma[round((S+1)/2),2]=1
      gamma[,3]=sqrt(exp(-seq(1,S,1)))
      
      # Pre-allocate M-vectors for the statistic and B=1 bootstrap statistic per monte carlo rep, for Warp Speed bootstrap
      statmatrix=matrix(NA,M,NDGP)
      bootstatmatrix=matrix(0,M,(NDGP*nrow(rq)))
      
      # Loop over Monte Carlo replications
      for (mm in 1:M){
        
        # Draw Z's
        Z=matrix(rnorm(T*S),T,S)
        # Calculate variance of epsilon
        sigmaeps=sqrt(colSums(gamma^2))
        # Draw epsilon under DGPZ-N, DGPZ-A1 and DGPZ-A2
        epsyN=matrix(rnorm(T,0,sigmaeps[1]),T,1)
        epsyA1=matrix(rnorm(T,0,sigmaeps[2]),T,1)
        epsyA2=matrix(rnorm(T,0,sigmaeps[3]),T,1)
        
        yrawN=Z%*%matrix(gamma[,1])+epsyN
        yrawA1=Z%*%matrix(gamma[,2])+epsyA1
        yrawA2=Z%*%matrix(gamma[,3])+epsyA2
        
        # Pre-allocate out-of-sample nowcast prediction matrices
        oosN=matrix(NA,P,S)
        oosA1=matrix(NA,P,S)
        oosA2=matrix(NA,P,S)
       
        # Loop over out-of-sample points (quarters)
        for (t in 1:P){
          # Loop over nowcast points (within quarter)
          for (i in 1:S){
          
            # Run regression of y_t only Z_i
            Zt=cbind(matrix(1,(R+t-1),1),Z[1:(R+t-1),i])
            
            betaN=solve(t(Zt)%*%Zt)%*%t(Zt)%*%matrix(yrawN[1:(R+t-1),])
            betaA1=solve(t(Zt)%*%Zt)%*%t(Zt)%*%matrix(yrawA1[1:(R+t-1),])
            betaA2=solve(t(Zt)%*%Zt)%*%t(Zt)%*%matrix(yrawA2[1:(R+t-1),])
            
            # Make nowcast of y_t and store in oos. This uses the forecast values of X_t and y_t-1 if it is not yet available
            oosN[t,i]=t(betaN)%*%matrix(c(1,Z[(R+t),i]))
            oosA1[t,i]=t(betaA1)%*%matrix(c(1,Z[(R+t),i]))
            oosA2[t,i]=t(betaA2)%*%matrix(c(1,Z[(R+t),i]))
            
          }
          
          # t loop end  
        }
        
        # Derive nowcast errors
        epsN=matrix(yrawN[(R+1):T,])%*%matrix(1,1,S)-oosN
        epsA1=matrix(yrawA1[(R+1):T,])%*%matrix(1,1,S)-oosA1
        epsA2=matrix(yrawA2[(R+1):T,])%*%matrix(1,1,S)-oosA2
      
        # Obtain squared error series, stacked from left to right
        eps2=cbind(epsN^2,epsA1^2,epsA2^2)  
        
        # Charts of MSFE (keep switched off when looping)
        #fbarN=colMeans(epsN^2)
        #plot(fbarN,type="l")
        #fbarA1=colMeans(epsA1^2)
        #plot(fbarA1,type="l")
        #fbarA2=colMeans(epsA2^2)
        #plot(fbarA2,type="l")
        
        # Generate the moment conditions upon which the test statistic is based
        # Note that these are MSFE differentials
        # As in paper, k_S^c is used to separate inequalities some distance apart
        
        # Set ineq = 1,...,5 where:
        # 1 = full set of inequalities
        # 2 = k_S^c=1
        # 3 = k_S^c=2
        # 4 = k_S^c=5
        # 5 = P&T Delta^e (adjacent-only)
        
        for (t in 1:NDGP){
          
          # Select relevant squared error series from eps2
          ff=eps2[,((t-1)*S+1):(t*S)]
          
          if (ineq<5){
            f=matrix(NA,P,0)
            spacing=c(0,1,2,3)[ineq]
            for (j in (1+spacing):(ncol(ff)-1)){
              f=cbind(f,(ff[,(1+j):ncol(ff)]-ff[,1:(ncol(ff)-j)]))
            }
            
            KK=((SS-spacing)*(SS-spacing-1)/2) 
          }
          
          # Adjacent-only version
          if (ineq==5){
            f=ff[,2:ncol(ff)]-ff[,1:(ncol(ff)-1)]
            KK=SS-1
          } 
          
          # Define number of moment inequalities
          kappa=ncol(f)
          
          if (t==1) {
            fN = f 
          } else if (t==2){
            fA1 = f
          } else if (t==3){
            fA2 = f
          } 
          
        }
        
        # Generate matrix of means fbar, stacked as a (3 x kappa) matrix with 
        # rows corresponding to DGPZ-N, DGPZ-A1, DGPZ-A2
        fbar=matrix(0,NDGP,kappa)
        fbar[1,]=matrix(colMeans(fN),1,kappa)
        fbar[2,]=matrix(colMeans(fA1),1,kappa)
        fbar[3,]=matrix(colMeans(fA2),1,kappa)
       
        # Test statistics for DGP-N and DGP-A1 to DGP-A4
        # "apply" function takes the max over fbar row-by-row.
        U=sqrt(P)*matrix(apply(fbar,1,max),1,NDGP)
        statmatrix[mm,]=U
        
        # Bootstrap ---------------------------------------------------------------
        
        for (bb in 1:nrow(rq)){
          
          r=rq[bb,1]
          q=rq[bb,2]
          
          # m taken to give largest integer below P of the form m*(r+q)
          m=floor(P/(r+q))
          
          # Preallocate temporary matrices for the large blocks for fN,fA1,fA2
          tempN=matrix(0,(q*m),kappa)
          tempA1=matrix(0,(q*m),kappa)
          tempA2=matrix(0,(q*m),kappa)
  
          # Small block/large block boostrap forms blocks of length (q+r) and drops the last small block of length r
          # In each block of length (q+r) from fN,...,fA2.
          # The mean of the original variables is subtracted prior to multiplying by the random draw	
          
          # Extract blocks from the matrices fN,fA1,fA2	into temp and recentre around the sample mean
          # This is used to create the bootstrapped fn, fA1, fA2
          for (i in 1:m){
            tempN[((i-1)*q+1):(i*q),]=(fN[((i-1)*(q+r)+1):(i*(q+r)-r),]-matrix(fbar[1,],q,kappa,byrow=TRUE))
            tempA1[((i-1)*q+1):(i*q),]=(fA1[((i-1)*(q+r)+1):(i*(q+r)-r),]-matrix(fbar[2,],q,kappa,byrow=TRUE))
            tempA2[((i-1)*q+1):(i*q),]=(fA2[((i-1)*(q+r)+1):(i*(q+r)-r),]-matrix(fbar[3,],q,kappa,byrow=TRUE))
          }
          
          # Bootstrap resample (B=1 for warp speed bootstrap)
          
          # Draw an (m x 1) random i.i.d.Normal vector for multiplier bootstrap
          # Repeat each draw m times to do element-wise multiplication
          random=matrix(rnorm(m,0,1),m,1)	     
          Eps=matrix(rep(random,each=q),nrow=length(rep(random,each=q)),ncol=kappa,byrow=FALSE) 	
          
          # The (q*m)-matrix extracted above is pre-multiplied by the random normal draw contained in the (q*m)x(kappa) matrix Eps	
          fNstar = Eps*tempN
          fA1star= Eps*tempA1
          fA2star= Eps*tempA2
         
          # Bootstrap fbar
          fbarstar=matrix(0,NDGP,kappa)
          fbarstar[1,]=matrix(colMeans(fNstar),1,kappa)
          fbarstar[2,]=matrix(colMeans(fA1star),1,kappa)
          fbarstar[3,]=matrix(colMeans(fA2star),1,kappa)
          
          # Bootstrap max statistics  
          bootstatmatrix[mm,(NDGP*(bb-1)+1):(NDGP*bb)]=sqrt(m*q)*matrix(apply(fbarstar,1,max),1,NDGP)
          
          # Block combinations bb loop end
        }
        
        if(mm==1){
          cat(paste("/// Progress: ",(nrow(SS)*(pp-1)+ss)," of ",(nrow(SS)*nrow(PP))," ",sep=""))
        } else{}
        if (M>1){
          if ((mm %in% seq(10,10*floor(M/10),10))==TRUE){
            cat(mm)
          } else{
            cat('.')
          }  
        }
        
        # Monte Carlo mm loop end
      }
      
      # Work out rejection rate
      for (j in 1:nrow(rq)){
        for (i in 1:NDGP){
          
          results[(nrow(rq)*(pp-1)+j),(NDGP*(ss-1)+i)]=colSums(matrix(statmatrix[,i]>matrix(sort(bootstatmatrix[,(NDGP*(j-1)+i)]))[((1-nomsize)*(M+1)),]))/M
      
        }
      }
      
      # S sample size ss loop end
    }
    
    # pi sample size pp loop end  
  }
  
  # Pre-allocate empty results table
  table=matrix(NA,0,(ncol(results)+2))
  
  # Create tables with row/column headings
  for (pp in 1:nrow(PP)){
    table=rbind(table,rbind(cbind(matrix(c(""),1,2),matrix(c(paste("(R,P)=(",round(T/(1+PP[pp])),",",(T-round(T/(1+PP[pp]))),")",sep="")),1,(NDGP*nrow(SS)))),cbind(matrix(c(""),1,2),matrix(c(paste("kap=",kronecker(matrix(KK[,1]),matrix(1,NDGP,1)),sep="")),1,(NDGP*nrow(SS)))),cbind(matrix(c(""),1,2),matrix(c("DGP-N","DGP-A1","DGP-A2"),1,(NDGP*nrow(SS)))),cbind(matrix(paste("r=",rq[,1],sep="")),matrix(paste("q=",rq[,2],sep="")),results[(nrow(rq)*(pp-1)+1):(pp*nrow(rq)),])))
  }
  
  # Save csv of results
  path=paste("Results-PEEX-T-",T,"-spacing-",c(0,1,2,3,"PT")[ineq],".csv",sep="")
  write.table(table,path, row.names=FALSE, col.names=FALSE, sep=",")
  
  # ineq loop end
}

cat(paste("Running Time: ",floor(as.matrix(proc.time()-time)[3,]/60),"mins ",round((as.matrix(proc.time()-time)[3,]-60*floor(as.matrix(proc.time()-time)[3,]/60))),"sec",sep=""))